%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This file is part of QTempIntMiner.
%
% QTempIntMiner is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% QTempIntMiner is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% Affero GNU General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with QTempIntMiner.  If not, see <http://www.gnu.org/licenses/>.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [Mr, owner] = AP_TemporalPatternConstruction(itemset, itemloc)
%
% Author : Guyet T. <thomas.guyet@irisa.fr>


    global distance_function; %Temporal intervals distance
    
    p_dim=size(itemset, 2);
    
    %% construct the set of intervals
    Intervals=[];
    for ide = 1:size(itemloc,2)
        e= itemloc{ide}.is;
        
        I=zeros(p_dim,2);
        I(:,1)=e(2,:)';
        I(:,2)=e(3,:)'+I(:,1);
        if( isempty(Intervals) ) 
            Intervals = I;
        else
            Intervals=cat(3,Intervals,I);
        end
    end
    
    %Construction de la matrice de distances pour AP
    N=size(Intervals, 3);
    M=N*N-N;
    s=zeros(M,3);
    j=1;
    for iI=1:N
      for k=[1:iI-1,iI+1:N]
        s(j,1)=iI;
        s(j,2)=k;
        I1=Intervals(:,:,iI);
        I2=Intervals(:,:,k);
        
        distfn=str2func(distance_function);
        s(j,3)=-distfn(I1, I2);
        j=j+1;
      end
    end
    p=median(s(:,3));
    
    %% cluster the intervals using Affinity Propagation algorithm
    %fprintf('Launch AP (p=%g)\n', p);
    [owner]=apcluster(s,p);
    %fprintf('\t#clusters: %d\n', size(unique(owner),1));
    
    %% new temporal itemset construction
    Mr=[];
    for id_cluster=unique(owner)'
        new_itemset=itemset;
        new_itemset(2,:)=Intervals(:, 1, id_cluster)';
        new_itemset(3,:)=Intervals(:, 2, id_cluster)' - Intervals(:, 1, id_cluster)';
        if( isempty(Mr) )
            Mr=new_itemset;
        else
            Mr=cat(3, Mr, new_itemset);
        end
    end
