%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This file is part of QTempIntMiner.
%
% QTempIntMiner is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% QTempIntMiner is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% Affero GNU General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with QTempIntMiner.  If not, see <http://www.gnu.org/licenses/>.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function P=GenProto(Size, NbSymb, VarDateMax, DateMax, VarDurationMax, DurationMax)
% Random pattern prototype generation
%
% Usage
% 	Size : Create pattern prototype size, ie the number of symbols in the pattern
% 	prototype, required.
% 	NbSymb : Nb of available symbols, required.
% 	VarDateMax : Maximum variance of the dates, default 2.
%	DateMax : Maximum date value, default 15.
%	VarDurationMax : Maximum Variance of the durations, default
%	VarDurationMax=VarDateMax/Size.
%	DurationMax : Maximum duration value, default DurationMax=DateMax/Size.
%
%	returns P, a pattern prototype (matrice of size Size x 5)
%       P(:,i) is a symbol prototype :
%            P(1,i) is the symbolic name of the symbol prototype
%            P(2,i) is the mean date of the symbol prototype
%            P(3,i) is the date variance of the symbol prototype
%            P(4,i) is the mean duration of the symbol prototype
%            P(5,i) is the duration variance of the symbol prototype
%   A pattern prototype is used to generate sequence with the above
%   statistical properties
%
% Author : Guyet T. <thomas.guyet@irisa.fr>

if nargin<2
   disp('Nombre d`arguments insuffisant');
   return;
end

% Default parameters
if nargin < 3, VarDateMax=2; end
if nargin < 4, DateMax=15; end
if nargin < 5, VarDurationMax=VarDateMax/Size; end
if nargin < 6, DurationMax=DateMax/Size; end


P=zeros(5,Size);

P(1,:)=floor(NbSymb*rand([1,Size]))+1;

P(2,:)=DateMax*rand([1,Size]);
P(2,:)=sort(P(2,:));

P(3,:)=VarDateMax*rand([1,Size]);

for i=1:(Size-1)
   P(4,i)=rand(1)*(P(2,i+1)-P(2,i));
end
P(4,Size)=rand(1)*(DateMax-P(2,Size));

P(5,:)=VarDurationMax*rand([1,Size]);
