%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This file is part of QTempIntMiner.
%
% QTempIntMiner is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% QTempIntMiner is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% Affero GNU General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with QTempIntMiner.  If not, see <http://www.gnu.org/licenses/>.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function S=GenScenAlea(Size, NbSymb, VarDateMax, DateMax, VarDurationMax, DurationMax)
% Random generation of a pattern prototype
%
% Usage:
%   - Size: Nb of symbols in the sequence generated, required.
%   - NbSym: Nb of available symbols, required.
%   - VarDateMax, see GenProto, default 1
%   - DateMax, see GenProto, default 15.
%   - VarDurationMax, see GenProto, default VarDateMax/Size.
%   - DurationMax, see GenProto, default DurationMax=DateMax/Size.
%
% The function uses the GenProto function and extracts only the interesting
% part of the protptype to construct a pattern.
%   
% Author : Guyet T. <thomas.guyet@irisa.fr>


if(nargin<2)
   error('Not enough arguments');
   return;
end

%Default parameters
if nargin < 3, VarDateMax=1; end
if nargin < 4, DateMax=15; end
if nargin < 5, VarDurationMax=VarDateMax/Size; end
if nargin < 6, DurationMax=DateMax/Size; end

P=GenProto(Size, NbSymb, VarDateMax, DateMax, VarDurationMax, DurationMax);
S=P([1,2,4],:);

