%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This file is part of QTempIntMiner.
%
% QTempIntMiner is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% QTempIntMiner is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% Affero GNU General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with QTempIntMiner.  If not, see <http://www.gnu.org/licenses/>.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [Ex,ok]=ReadExamples(filename)
% Read a filename of examples
% Usage
%   - Ex : Example base  (array of cells)
%		Ex{i} holds the i-th example : an array of size 3xn where n is the number of item in the sequence
%   - filename : name of the file to write
%
% Returns : return 0 if everything was alright, and otherwise
% an error occurred
%
% Author : Guyet T. <thomas.guyet@irisa.fr>

if( ~ischar(filename) )
	ok=2;
	error('ReadExample error', 'Wrong filename');
	return;	
end

%open file
[fid, message] = fopen(filename,'r');

if (fid==-1 )
	ok=1;
	error('ReadExample error', strcat('fileopen error : ', message));	
	return;
end

Ex=cell(1,1);
idEx=1;
dim=1;
tline=fgetl(fid);
ex=zeros(1,3);
while ischar(tline)
	if(tline(1)=='=')
		Ex{idEx} = ex;
		idEx=idEx+1;
		ex=zeros(1,3);
		dim=1;
	else
		A=sscanf(tline, '%f %f %f')
		assert( size(A,1)==3, 'Wrong example format');
		ex(dim,:)=A';
		dim=dim+1;
	end
    tline = fgetl(fid);
end

fclose(fid);
ok=0;

