%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This file is part of QTempIntMiner.
%
% QTempIntMiner is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% QTempIntMiner is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% Affero GNU General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with QTempIntMiner.  If not, see <http://www.gnu.org/licenses/>.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function ok=WriteExamples(Ex, filename)
% Generate NbEx examples of a random pattern prototype
% Usage
%   - Ex : Example base  (array of cells)
%		Ex{i} holds the i-th example : an array of size 3xn where n is the number of item in the sequence
%   - filename : name of the file to write
%
% Returns : return 0 if everything was alrigth, and otherwise
% an error occurred
%
% Author : Guyet T. <thomas.guyet@irisa.fr>


if( ~ischar(filename) )
	ok=2;
	error('WriteExample error', 'Wrong filename');
	return;	
end

%open file
[fid, message] = fopen(filename,'w');

if (fid==-1 )
	ok=1;
	error('WriteExample error', strcat('fileopen error : ', message));	
	return;
end

for(i=1:length(Ex))
	e=Ex{i};
	assert( size(e,1)==3, 'Wrong example format');
	for( j=1:size(e, 2) )
		%one item
		fprintf(fid,'%d %f %f\n', e(1, j), e(2, j), e(3, j) );
	end
	%end the current sequence :
	if( i~=length(Ex) )
		fprintf(fid,'==\n');
	end
end

fclose(fid);
ok=0;

