%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This file is part of QTempIntMiner.
%
% QTempIntMiner is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% QTempIntMiner is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% Affero GNU General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with QTempIntMiner.  If not, see <http://www.gnu.org/licenses/>.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [Mr, owner] = DynaClust_TemporalPatternConstruction(itemset, itemloc)
%
% Author : Guyet T. <thomas.guyet@irisa.fr>


    global nbClusterMax;        % Max number of cluster to test

    p_dim=size(itemset, 2);
    
    %% construct the set of intervals
    Intervals=[];
    for ide = 1:size(itemloc,2)
        e= itemloc{ide}.is;
        
        I=zeros(p_dim,2);
        I(:,1)=e(2,:)';
        I(:,2)=e(3,:)'+I(:,1);
        if( isempty(Intervals) ) 
            Intervals = I;
        else
            Intervals=cat(3,Intervals,I);
        end
    end
    
    %% cluster the intervals
    max_sqrerr=inf;
    fprintf('Launch ACPA (MIN SQRERR, Weighted Hausdorff)\n');
    for nb_clusters = 1:min(nbClusterMax, size(Intervals,3) )
        lambda = ones(size(Intervals,1), 1);
        [kcentre, ~, owner, sqrerr] = APCA(Intervals, nb_clusters, lambda, 'hausdorff'); % autre distance : cityblock
        fprintf('SQRERR %g\n', sqrerr);
        if( sqrerr < max_sqrerr )
            max_sqrerr=sqrerr;
            kcentre_opt=kcentre;
        end
    end
    
    %% new temporal itemset construction
    Mr=[];
    for id_cluster=1:size(kcentre_opt,3)
        new_itemset=itemset;
        new_itemset(2,:)=kcentre_opt(:,1, id_cluster)';
        new_itemset(3,:)=kcentre_opt(:,2, id_cluster)'-kcentre_opt(:,1, id_cluster)';
        if( isempty(Mr) )
            Mr=new_itemset;
        else
            Mr=cat(3, Mr, new_itemset);
        end
    end
