% Ichino & Yaguchi "distance" between intervals
% T. Guyet
% - I1, I2 : intervals to compare
% - gamma : value between 0 and 0.5 (default 0.25)
% - normalisaion : 'DeCarvahloDiday', 'IchinoYaguchi' or 'none' (default 'none')
% 
function d=IchinoYaguchi(I1, I2, gamma, normalisation)

d=0;
if( size(I1,1)~=size(I2,1) )
    return;
end

try
    if( isempty(gamma) || gamma <0 || gamma >0.5 )
        gamma=0.25;
    end
catch ME
    %ME.message
    gamma=0.25;
end

try
    if( isempty(normalisation) )
        normalisation='none';
    end
catch ME
    %ME.message
    normalisation='none';
end


n=size(I1,1);

d=0;
for i=1:n    
    envelop = max(I1(i,2), I2(i,2)) - min(I1(i,1),I2(i,1));
    inter=max(0, min(I1(i,2), I2(i,2)) - max(I1(i,1),I2(i,1)));
    muI1 = (I1(i,2)- I1(i,1));
    muI2 = (I2(i,2)- I2(i,1));
    
    dj= envelop - inter + gamma * ( 2*inter - muI1 - muI2);

    if( strcmp(normalisation,'DeCarvahloDiday') )
        dj = dj / envelop;
    elseif( strcmp(normalisation,'IchinoYaguchi') ) 
        Dj = (I1(i,2)- I1(i,1)) + (I2(i,2) -I2(i,1));
        dj = dj / Dj;
    end
    d = d + dj;
end
%d = d/n;
