% Ichino & Yaguchi "distance" between intervals
% T. Guyet
% - I1, I2 : intervals to compare
% - gamma : value between 0 and 0.5 (default 0.25)
% - normalisaion : 'DeCarvahloDiday', 'IchinoYaguchi' or 'none' (default 'none')
% 
function d=IchinoYaguchiCombined(I1, I2, gamma, normalisation, w, q)

d=0;
n=size(I1,1);
try
    if( n~=size(I2,1) || size(w,1)~=n)
        disp('IchinoYaguchiCombined error');
        return;
    end
    if( sum(w)~=1 || min(w)<0 )
        w=ones(n, 1)/n;
    end
catch ME
    %ME.message
    w=ones(n, 1)/n;
end

try
    if( isempty(gamma) || gamma <0 || gamma >0.5)
        gamma=0.25;
    end
catch ME
    %ME.message
    gamma=0.25;
end

try
    if( isempty(normalisation) )
        normalisation='none';
    end
catch ME
    %ME.message
    normalisation='none';
end

try
    if( isempty(1) || q<1)
        q=1;
    end
catch ME
    %ME.message
    q=1;
end

d=0;
for i=1:n    
    envelop = max(I1(i,2), I2(i,2)) - min(I1(i,1),I2(i,1));
    inter=max(0, min(I1(i,2), I2(i,2)) - max(I1(i,1),I2(i,1)));
    muI1 = (I1(i,2)- I1(i,1));
    muI2 = (I2(i,2)- I2(i,1));
    
    dj= envelop - inter + gamma * ( 2*inter - muI1 - muI2);

    if( strcmp(normalisation,'DeCarvahloDiday') )
        dj = dj / envelop;
    elseif( strcmp(normalisation,'IchinoYaguchi') ) 
        Dj = (I1(i,2)- I1(i,1)) + (I2(i,2) -I2(i,1));
        dj = dj / Dj;
    end
    d = d + w(i,1)*dj^q;
end
d = d^(1/q);
%d = d/n;
    