
function [kcentre,owner,sqrerr,bic] = gmeans_intervals(data, initialcentres, alpha)
%
% [kcentre,owner,sqrerr, bic] = gmeans_intervals(data, initialcentres)
%
%
% Input:
%   data = a matrix of size NxLx2 (N=number of data points,
%   L=dimensionality) : intervals (begin, end)
%   initialcentres = scalar value (i.e. the 'K' in K-means, how many clusters)
%                    or a KxL matrix, defining the initial centres
%   alpha : significance for the Anderson-Darling test
%
% Output:
%   kcentre = KxL matrix of final cluster centres
%   owner = Nx1 vector indicating which cluster a point belongs to
%   sqrerr = sum of squared errors (squared distances from nearest cluster centre)
%   bic = bayes information criterion
%

N = size(data,3); % num intervals
L = size(data,1); % num variables
bic=inf;

if (max(size(initialcentres))==1)
    % I've been given the scalar value K...setup centres at first K data points
    %K = initialcentres;
    initialcentres = data(:,:,1:min(size(data,3),initialcentres));
end

if (size(initialcentres,3) ~= 2 || size(initialcentres,1) ~= L)
    error('wrong size matrix: initialcentres');
end

K = size(initialcentres,3); % num clusters

kcentre = initialcentres; % L * K * 2
ownerchange = 1;
owner = zeros(N,1);
while (ownerchange > 0)
    % Do clustering
    ownerchange = 0;
    for i = 1:N
        % find distance to each cluster centre
        best = 0;
        bestdist = inf;
        for j = 1:K
            %dist=Hausdorff(data(:,:,i), kcentre(:,:,j));
            dist=CityBlock(data(:,:,i), kcentre(:,:,j));
            %dist=GowdaDiday(data(:,:,i), kcentre(:,:,j));
            %dist=IchinoYaguchi(data(:,:,i), kcentre(:,:,j), 0.25, 'none');
            %dist=DeCarvahlo94(data(:,:,i), kcentre(:,:,j));
            
            if( dist < bestdist )
                best=j;
                bestdist=dist;
            end
        end

        % pick closest centre, and note if a point changes cluster
        if( owner(i)~=best )
            owner(i)=best;
            ownerchange=1;
        end
    end

    % Recalculate cluster centres for each cluster
    for i = 1:K
        total = sum(owner==i);
        % for each variable
        if (total > 0)
            kcentre(:,:,i) = median( data(:,:,owner==i), 3 );
        else
            % This cluster has no members!
            error('empty cluster');
            %Might generate a random cluster :
            %kcentre(i,:) = data(floor(rand*N)+1,:)+(10^-10)*randn(1,L);
        end
    end

end


%% clusters separation
newcenters=[];
if( L==1 )
    %todo separate the cluster !
else
    for c = 1:K
        pos=data(:,1,owner==c);
        x=reshape(pos, size(pos,3), 1);
        
        %Anderson-darling test for cluster c
        s=std(x);
        mu=mean(x);
        %Normalisation des donn?es
        x = (x-mu*ones(1, size(x,3)))./s;
        x = sort(x);
        z = 0.5*(1+erf(x));
        A2=0;
        for i=1:N
            A2 = A2 + (2*i-1)*( log( z(i,1) ) + log( 1-z(N+1-i,1) ) );
        end
        A2=A2/N;
        A2=A2-N;
        
        if( isempty(newcenters) )
            newcenters = kcentre(:,:,c);
        else
            newcenters = cat(3, newcenters, kcentre(:,:,c) );
        end
        
        %Duplicate the center if Ho is refused
        if( A2 > alpha ) 
            newcenters = cat(3, newcenters, kcentre(:,:,c) );
        end
        
    end
end

