%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This file is part of QTempIntMiner.
%
% QTempIntMiner is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% QTempIntMiner is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% Affero GNU General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with QTempIntMiner.  If not, see <http://www.gnu.org/licenses/>.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [owner, Separations, interval_envelope] = Intervals_Separation(Intervals)
%
% Author : Guyet T. <thomas.guyet@irisa.fr>
% GNU Licenced, 2013
%
% Find separations between intervals
%
% Intervals matrix of size n x 2, where n is the number of intervals and I(i,1) is the 
% 	start date of the interval i and I(i,2) is the end date of the interval i
%
% return owner, a vector of indexes of the Intervals "clusters".
%		Separations a matrix of intervals giving the separations between clusters (ordered). 
%		interval_envelope : the envelope of the whole intervals
%
	Separations=[];
    owner=[];
    interval_envelope = [];

    if size(Intervals,1)<1
        %no separation possible
        return
    end
    
    
    interval_envelope=Intervals(1,:); %first interval holds the envelope of the intervals
    
    for ide = 2:size(Intervals,1)
        I=Intervals(ide,:);
        if isempty(Separations) %tant qu'il n'y a pas de separation on fait grossir l'interval
                %create the first separation
            if interval_envelope(2)<I(1) % not overlap
                Separations = [interval_envelope(2) I(1)];
            elseif I(2)<interval_envelope(1)
                Separations = [I(2) interval_envelope(1)];
            end
        else %ici, on a deja des separations
            if interval_envelope(2)<I(1) % not overlap
                S = [interval_envelope(2) I(1)];
                Separations=cat(1, Separations, S);
            elseif I(2)<interval_envelope(1)
                S = [I(2) interval_envelope(1)];
                Separations=cat(1, Separations, S);
            else % il y a overlap avec l'envelope, donc possiblement avec les separations
                %on reduit chacune des separations
                
                i=1;
                while i < size(Separations,1)
                    S=Separations(i,:);
                    if ~(S(2)<I(1) || S(1)>I(2)) %overlap
                        if S(2)>I(2) && S(1)<I(1) %inclusion of I in S : two separations
                            Separations(i,2)=min(S(2), I(1));
                            S(1)=I(2);
                            Separations=cat(1, Separations, S);
                        elseif I(2)>S(2) %overlap on the right of the S interval
                            Separations(i,2)=I(1);
                        elseif I(1)<S(1) %overlap on the left of the S interval
                            Separations(i,1)=I(2);
                        end
                        if Separations(i,1)>Separations(i,2)
                            Separations(i,:)=[]; %deleting the separation
                        end
                    end
                    i=i+1;
                end
            end
        end
        %update the overall envelope
        interval_envelope=[ min(interval_envelope(1), I(1)) max(interval_envelope(2), I(2))];
    end

 	%% construction the owner vector
    owner=ones(size(Intervals,1),1);
    if ~isempty(Separations)
		% order Separations
		Separations = sortrows(Separations, 1);
	  
	  	% assign Intervals to "clusters"
		for ide = 1:size(Intervals,1)
		    for i = size(Separations,1):-1:1
		        if Intervals(ide,2)>Separations(i,1)
		            owner(ide)=i+1;
		            break;
		        end
		    end
		end
	end
	
%end function
end
