%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This file is part of QTempIntMiner.
%
% QTempIntMiner is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% QTempIntMiner is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% Affero GNU General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with QTempIntMiner.  If not, see <http://www.gnu.org/licenses/>.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [Mr, owner] = KMeans_TemporalPatternConstruction(itemset, itemloc)
%
% Author : Guyet T. <thomas.guyet@irisa.fr>
%


    global distance_function;   % Name of a distance function
    global nbClusterMax;        % Max number of cluster to test
    global bestkmeans_selection_method; % Nb clusters selection method
    if(isempty(bestkmeans_selection_method)), bestkmeans_selection_method='BIC'; end

    p_dim=size(itemset, 2);
    
    %% construct the set of intervals
    Intervals=[];
    for ide = 1:size(itemloc,2)
        e= itemloc{ide}.is;
        
        I=zeros(p_dim,2);
        I(:,1)=e(2,:)';
        I(:,2)=e(3,:)'+I(:,1);
        if( isempty(Intervals) ) 
            Intervals = I;
        else
            Intervals=cat(3,Intervals,I);
        end
    end
    
    %% cluster the intervals
    switch( bestkmeans_selection_method )
        case 'SQRERR'
            max_sqrerr=inf;
            %fprintf('Launch KMeans (MIN SQRERR)\n');
            for nb_clusters = 1:min(nbClusterMax, size(Intervals,3) )
                [kcentre, owner, sqrerr, ~] = kmeans_intervals(Intervals, nb_clusters, distance_function);
                %fprintf('BIC %g SQRERR %g\n', bic, sqrerr);
                if( sqrerr < max_sqrerr )
                    max_sqrerr=sqrerr;
                    kcentre_opt=kcentre;
                end
                if( isnan(sqrerr) )
                    %fprintf('un seul intervalle\n');
                    kcentre_opt=kcentre;
                end
            end
        case 'BIC'      % X-Means strategy
            min_bic=inf;
            %fprintf('Launch KMeans (MIN BIC)\n');
            for nb_clusters = 1:min(nbClusterMax, size(Intervals,3) )
                [kcentre, owner,~, bic] = kmeans_intervals(Intervals, nb_clusters, distance_function);
                %fprintf('BIC %g SQRERR %g\n', bic, sqrerr);
                if( bic <= min_bic )
                    min_bic=bic;
                    kcentre_opt=kcentre;
                end
                if( isnan(bic) )
                    %fprintf('un seul intervalle\n');
                    kcentre_opt=kcentre;
                end
            end
        case 'GMeans'   % G-Means strategy
            %fprintf('Launch GMeans\n');
            [kcentre_opt] = gmeans_intervals(Intervals, 1);
        otherwise
            fprintf('KMeans Interval error\n');
            return;
    end
    
    %% new temporal itemset construction
    Mr=[];
    for id_cluster=1:size(kcentre_opt,3)
        new_itemset=itemset;
        new_itemset(2,:)=kcentre_opt(:,1, id_cluster)';
        new_itemset(3,:)=kcentre_opt(:,2, id_cluster)'-kcentre_opt(:,1, id_cluster)';
        if( isempty(Mr) )
            Mr=new_itemset;
        else
            Mr=cat(3, Mr, new_itemset);
        end
    end
