%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This file is part of QTempIntMiner.
%
% QTempIntMiner is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% QTempIntMiner is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% Affero GNU General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with QTempIntMiner.  If not, see <http://www.gnu.org/licenses/>.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function P=APriori(f, Ex, d, itMax)
% APriori main function
% [P]=APriori(f, E, d, itMax)
%	f : pattern frequence threshold (in [0,1])
%	Ex : Examples (cells that hold sequences). 
%		An sequence is an example : it must be a matrice (Ex) of size n*3, where n is the number of symbol in the sequence.
%		Ex(i,:) is the i-th symbol of the example Ex
%			Ex(i,1) is the symbolic name of the i-th symbol
%			Ex(i,2) is the date of the i-th symbol
%			Ex(i,3) is the duration of the i-th symbol
%			
%	d : density (histogram creation parameters)
%   itMax : Maximal number of iterations
%  returns 
%       P : the extracted patterns
%
%  In this algorithm, we first apply the APriori algorithm and then we
%  looked for quantitative time interval values
%
% Author : Guyet T. <thomas.guyet@irisa.fr>


%% Creation of the list of frequent symbols and associated patterns of size 1
Cs=patterns1D(f,Ex);
listSymbols=zeros(size(Cs,3),1);
for i=1:size(Cs,3)
    listSymbols(i)=Cs(1,1,i);
end
Cs=Cs(1,:)';

%% Main loop
k=1;
while(~isempty(Cs) && k<itMax)
    P=[];
    %% Generation of new candidats
    for i=1:size(Cs,1)
        for j=1:size(listSymbols,1)
          nm=zeros(1,k+1);
          nm(1,1:k)=Cs(i,:);
          nm(1,k+1)=listSymbols(j);
          % Checks the APriori constraint (None Here ... this might be improved)
          if( 1 )
             %Add nm to P
             if(~isempty(P))
                P=cat(1,P,nm);
             else
                P=nm;
             end
          end
        end
    end
    
    if(isempty(P))
        break;
    end
    
    %% Pruning
    LExp={};
    M=[];
    for i=1:size(P,1)
        n=0;
        NEx=[];
        for nex=1:size(Ex,1)
            e=Ex{nex}(1,:)';
            pe=1;
            j=1;
            found=0;
            while( j<=size(e,1) && pe<=size(P,2))
                if( e(j) == P(i,pe) )
                    if( pe == size(P,2)) 
                        found=1;
                    end
                    pe=pe+1;
                end
                j=j+1;
            end
            if( found )
                n=n+1;
                if(~isempty(NEx))
                   NEx=cat(1,NEx,nex);
                else
                   NEx=nex;
                end
            end
        end
        
        if(n>f*size(Ex,1))
             if(~isempty(M))
                M=cat(1,M,P(i,:));
             else
                M=P(i,:);
             end
             LExp{size(LExp,2)+1}=NEx;
        end
    end
    
    if(isempty(M))
        break
    else
        LEx=LExp;
        Cs=M;
    end
    k=k+1;
end

%% Temporal quantitative interval construction
P=[];
for i=1:size(Cs,1)
    [h,freq]=createHistogram( Cs(i,:)',Ex,d );
    if(freq>=f)
        P=createPatterns(P,Cs(i,:)',h, freq, f);
    end
end



