%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This file is part of QTempIntMiner.
%
% QTempIntMiner is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% QTempIntMiner is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% Affero GNU General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with QTempIntMiner.  If not, see <http://www.gnu.org/licenses/>.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function P=PrefixSpan(f, Ex, tMax)
% PrefixSpan main function
% [P]=PrefixSpan(f, E, d, itMax)
%	f : pattern frequence threshold (in [0,1])
%	E : Examples (cells that hold sequences). 
%		An sequence is an example : it must be a matrice (Ex) of size n*3, where n is the number of symbol in the sequence.
%		Ex(i,:) is the i-th symbol of the example Ex
%			Ex(i,1) is the symbolic name of the i-th symbol
%			Ex(i,2) is the date of the i-th symbol
%			Ex(i,3) is the duration of the i-th symbol
%   tMax : Maximal length of itemsets (cut the recursion depth)
%  returns 
%       P : the extracted patterns
%
% Author : Guyet T. <thomas.guyet@agrocampus-ouest.fr>



%% Creation of the list of frequent symbols and associated patterns of size 1
Cs=patterns1D(f,Ex);
listSymbols=zeros(size(Cs,3),1);
for i=1:size(Cs,3)
    listSymbols(i)=Cs(1,1,i);
end
Cs=Cs(1,:)';

%Construct the initial location
initloc=cell([1,size(Ex,1)]);
for i=1:size(Ex,1)
    s.pos=0;
    s.ExId=i;
    s.is=[];
    initloc{i}=s;
end

%PrefixSpan recursion root
P=PrefixSpanRec(f, Ex, tMax, Cs, [], initloc);


