%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This file is part of QTempIntMiner.
%
% QTempIntMiner is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% QTempIntMiner is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% Affero GNU General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with QTempIntMiner.  If not, see <http://www.gnu.org/licenses/>.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function P=PrefixSpanRec(f, Ex, tMax, L1, prefix, prefixloc)
%	f : pattern frequence threshold (in [0,1])
%	Ex : Examples (cells that hold sequences). 
%		An sequence is an example : it must be a matrice (Ex) of size n*3, 
%       where n is the number of symbol in the sequence.
%		Ex(i,:) is the i-th symbol of the example Ex
%			Ex(i,1) is the symbolic name of the i-th symbol
%			Ex(i,2) is the date of the i-th symbol
%			Ex(i,3) is the duration of the i-th symbol
%			
%   tMax : Maximal length of itemsets
%   L1 : frequent items (can be postfixed)
%   prefix : the prefix to extend
%   prefixloc : structured locatisation of the prefixes : a cell containing
%   couples (ExId, pos)
%   
%   returns 
%       P : the extracted patterns
%
% Author : Guyet T. <thomas.guyet@irisa.fr>

P={};

if( size(prefix,2)>=tMax )  % Stop the recursion if the prefix length is 
                            % upper than tMax
    return;
end


%Try to extend the prefix with one of the frequent items
for i=1:size(L1,1)
    item = L1(i);   % item is the postfixed item
    nbis=0;         % nbis is the number of occurrences of {prefix+item}
    itemloc={};     % cells of locations for the postfixed item
    for loc=1:size(prefixloc,2)
        s=prefixloc{loc};
        pos = s.pos+1;
        e=Ex{s.ExId}; %e is a sequence in which prefix occurs before 
                      %the position pos
        
        %Looking for the item after the position of the prefix (pos) in the
        %sequence e
        found=0;
        while( pos<=size(e,2) )
            if( e(1,pos)==item )
                found=1;
                break;
            end
            pos=pos+1;
        end
        
        if(found)
            nbis=nbis+1;
            s.pos=pos;
            s.is = cat(2, s.is, e(:,pos)); %extends the matching pattern
            itemloc=[itemloc s];
        end
    end
    
    if( nbis>f*size(Ex,1) )
        % HERE {prefix + item} is a frequent sequence
        itemset = cat(2,prefix,item);
        ExtendedPatterns=PrefixSpanRec(f, Ex, tMax, L1, itemset, itemloc);
        P=[P ExtendedPatterns];
    end
end

if( isempty(P) )
    % Any extended patterns have been founded has frequent, then prefix may
    % be in the bordure
    P={prefix; prefixloc};
end
