%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This file is part of QTempIntMiner.
%
% QTempIntMiner is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% QTempIntMiner is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% Affero GNU General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with QTempIntMiner.  If not, see <http://www.gnu.org/licenses/>.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function P=QTIPrefixSpan(f, Ex, tMax, distance_threshold, StartLevel_TPruning)
% PrefixSpan main function
% [P]=PrefixSpan(f, E, d, itMax)
%	f : pattern frequence threshold (in [0,1])
%	E : Examples (cells that hold sequences). 
%		An sequence is an example : it must be a matrice (Ex) of size n*3, where n is the number of symbol in the sequence.
%		Ex(i,:) is the i-th symbol of the example Ex
%			Ex(i,1) is the symbolic name of the i-th symbol
%			Ex(i,2) is the date of the i-th symbol
%			Ex(i,3) is the duration of the i-th symbol
%	d : density (histogram creation parameters)
%   tMax : Maximal length of itemsets (cut the recursion depth)
%  returns 
%       P : the extracted patterns
%
% Author : Guyet T. <thomas.guyet@irisa.fr>


%% Parameters verification
if(nargin<4)
    error('QTIPrefixSpan : not enough arguments')
end
if (nargin<5), StartLevel_TPruning=0; end

%% global variables

global G_explored_patterns;     % The set of explored patterns
global prune_super_explored;    % Require to prune super explored patterns
global bestkmeans_selection_method; %'BIC' or 'SQRERR'
global nbClusterMax; % kMeans parameter
global d; %EM parameter
global TPC_algo; %Temporal construction algorithm choice ('KMeans', 'EM', 'DynaClust' or 'AP')
global distance_function; %Temporal intervals distance (functor)

%Default values for global parameters
if(isempty(prune_super_explored)), prune_super_explored=true; end
if(isempty(bestkmeans_selection_method)), bestkmeans_selection_method='BIC'; end
if(isempty(nbClusterMax)), nbClusterMax=4; end
if(isempty(d)), d=1; end
if(isempty(distance_function)), distance_function='CityBlock'; end
if(isempty(TPC_algo)), TPC_algo='KMeans'; end



G_explored_patterns={};
P=[];

%% Creation of the list of frequent symbols and associated patterns of size 1
% Construct the initial location based on the 1D patterns

liste=[];
initloc=cell([1, size(Ex,1)]);
for i=1:size(Ex,1)
    e=Ex{i};
    for j=1:size(e,2)
        %Add to the list without copy
        ok=0;
        for k=1:size(liste,1)
            if(liste(k,1)==e(1,j))
                ok=1;
                liste(k,2)=liste(k,2)+1;
                break;
            end
        end
        if(~ok)
            liste=[liste;e(1,j),1];
        end
    end
    
    s.pos=0;
    s.ExId=i;
    s.is=[];
    initloc{i}=s;
end
listSymbols=[];
for i=1:size(liste,1)
    if(liste(i,2)>f*size(Ex,2))
        listSymbols=[listSymbols; liste(i,1)];
    end
end


%PrefixSpan recursion root
P=QTIPrefixSpanRec(f, Ex, tMax, distance_threshold, StartLevel_TPruning, listSymbols, [], initloc);

