%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This file is part of QTempIntMiner.
%
% QTempIntMiner is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% QTempIntMiner is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% Affero GNU General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with QTempIntMiner.  If not, see <http://www.gnu.org/licenses/>.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [b,val]=comparePatterns(m1, m2, distance_function, threshold)
% Compare two patterns symbolically (same symbols) and temporally (according to the
% interval distance function)
% [b,val]=comparePatterns(m1,m2,distance_function, threshold)
%       m1, m2: the pattern to compare
%       distance_function : the name of a interval distance (e.g.
%       CityBlock)
%       threshold : an interval similarity threshold (inf if not given)
%
%   b indicates if two patterns are similar (1) or not (0). Two patterns 
%   are similars if their symbolic patterns (only the nth first symbols) 
%   are the same and if the intervals distance is lower than the treshold.
%   val the distance value the symbolic patterns are the same, otherwise 0.
%   The algorithm assumes that symbols are temporally sorted.
%
% Author : Guyet T. <thomas.guyet@irisa.fr>


if( nargin <4 || isempty(threshold)  )
    disp('comparePatterns : no threshold');
    threshold = inf;
end

b=0;
val=0;

if( size(m1,2)==size(m2,2) )
    
    I1(:,1)=m1(2,:)';
    I1(:,2)=m1(2,:)'+m1(3,:)';
    I2(:,1)=m2(2,:)';
    I2(:,2)=m2(2,:)'+m2(3,:)';

    %Cas des informations temporelles non-totales
    for i=1:size(I1,1)
        if( m1(1,i) ~= m2(1,i) )
            %Patterns are symbolically distincts
            return;
        end
        if( I1(i,2)==I1(i,1) || I2(i,2)==I2(i,1) )
            I1(i,:)=I2(i,:);% Il n'y a pas d'information temporelle a 
                            % prendre en compte sur cette dimension, on 
                            % force l'egalite
                            % On peut soit avoir que des 0 soit des valeurs
                            % (toutes egales)
        %Pas de else : cas normal traite par defaut
        end
    end
    
    distfn = str2func(distance_function);
    
    val = distfn(I1, I2);
    
    if( val < threshold )
        b=1;
    end
end
