%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This file is part of QTempIntMiner.
%
% QTempIntMiner is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% QTempIntMiner is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% Affero GNU General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with QTempIntMiner.  If not, see <http://www.gnu.org/licenses/>.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function Sr=complementWith(S, s, begin, theend)
% Function to complement an sequence with symbol s from begin to end
%
% Author : Guyet T. <thomas.guyet@irisa.fr>

if size(S,2)~=3 || iscell(S) || isstruct(S) || isempty(S)
    disp 'complementWith error : wrong S format (must be a matrix of size nx3)'
    return;
end

if nargin < 2
    disp 'complementWith not enough arguments'
    return;
end

%% parameters
if(nargin==2)
    begin=min( S(:,2) );
    theend=max( S(:,2) + S(:,3) );
end

Sr=S;

current=begin;

% We sort the sequence with respect to the date of beginning of the
% events
S=sortrows(S,2);

for i=1:size(S,1)
    if S(i,2) >= theend || current >= theend
        break;
    end
    
    if current >= S(i,2)
        current = max( current, S(i,2)+S(i,3) );
        continue;
    else
        %insertion
        Sr=cat(1, Sr, [s current S(i,2)-current]);
        current=S(i,2)+S(i,3);
    end
end

if ~(S(i,2) >= theend || current >= theend)
    Sr=cat(1, Sr, [s current theend-current]);
end



Sr=sortrows(Sr,2);
