%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This file is part of QTempIntMiner.
%
% QTempIntMiner is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% QTempIntMiner is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% Affero GNU General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with QTempIntMiner.  If not, see <http://www.gnu.org/licenses/>.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function Cs=generateCandidates(M,symbList, distance_function, threshold)
% Generation of new patterns of size size(M,1)+1 based on the patterns of M
% Cs=generateCandidates(M,symbList)
%	M : A patterns matrix 
%		size(M,1) : Number of patterns
%		size(M,2) : 1 (1: the pattern, 2: the cell of matching instances)
%	symbList : List of frequent symbols
%	return Cs, the list of generated patterns 
%
% Author : Guyet T. <thomas.guyet@irisa.fr>


n=size(M{1,1},2);
Cs=[];
for i=1:size(M,1)
   for j=1:size(symbList,1)
        %% Construct a pattern of size n+1 based on M{i,1} pattern
        nm=zeros(3,n+1);
        nm(:,1:n)=M{i,1};
        nm(1,n+1)=symbList(j);

        %% Checking the APriori constraint
        % case j=1 : The sub-pattern pat (first symbol of nm is suppressed)
        % is looked for in M
        ok=0;
        pat=nm(:,2:(n+1));
        for k=1:size(M,1)
            if( comparePatterns(M{k,1}, pat, distance_function, threshold) )
                ok=1;
                break;
            end
        end
        if(~ok) % next pattern
            continue;
        end

        % other cases : 
        % The s-th symbol of nm is suppressed and the sub-pattern is looked for in M
        % The last sub-motif is not take into account! Must be satisfy because M should have been generated from M
        for s=2:n-1
            ok=0;
            pat=nm(:,[1:(s-1),(s+1):(n+1)]);
            for k=1:size(M,1)
                if( comparePatterns(M{k,1}, pat, distance_function, threshold) )
                    ok=1;
                    break;
                end
            end
            if(~ok)
                break;
            end
        end
        if(~ok)
            continue;
        end
      
        %% Concatenate the constructed pattern
        Cs=[Cs; {nm} M(i,2)];
   end
end
