%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This file is part of QTempIntMiner.
%
% QTempIntMiner is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% QTempIntMiner is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% Affero GNU General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with QTempIntMiner.  If not, see <http://www.gnu.org/licenses/>.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [b, mindist] = similarSubPattern(m1, m2, distance_function, threshold)
% Function to compare m1 with a sub pattern of m2 according to a given interval distance
% function.
% [b, val] = similarSubPattern(m1, m2, distance_function, threshold)
%
%   If several alignment are possible, the function try all of them to give
%   the compare the lowest distance with the threshold.
%   Be careful, it's a quite tricky algorithm.
%
% input
%   m1, m2 : two patterns
%   distance_function : the name of a distance function between intervals
%   (eg CityBlock)
%   threshold : an interval similarity threshold (inf if not given)
%
% output:
%   b : 1 if m1 has been symbolically found in m2 and if the distance
%       between intervals are lower than the threshold
%   mindist : the value of the distance between intervals
%
% Author : Guyet T. <thomas.guyet@irisa.fr>


b=0;
mindist=inf;

if( nargin < 4)
    threshold=inf;
end

if( size(m1,2)>size(m2,2) )
    return;
end

c=ones(size(m1,2),1); %c is an index vector : c(i) gives the index of an item of m2 that is equals to the i-th item of m1

test=1;

while(test)
    config_found=1;
    for i=1:size(c,1)
        j=c(i);
        while( j<=size(m2,2) && m1(1,i)~=m2(1,j) )
            j=j+1;
        end
        if( j>size(m2,2) || m1(1,i)~=m2(1,j) ) %HACK possibilite de s'arreter lorsque j ne laisse plus suffisamment de place au reste du motif !
            config_found=0;
            break;
        end
        
        %On donne la bonne valeur a c(i)
        c(i)=j;
        
        % On repousse les suivants en utilisant l'ordre des items
        % If values have already been setted, we don't change it
        for k=(i+1):size(c,1)
            if( c(k)<c(k-1)+1 )
                c(k)=c(k-1)+1;
            end
        end
    end
    if( ~config_found )
        break;
    end
    % HERE : we found a good mapping between patterns, we compare the
    % corresponding intervals
    distfn=str2func(distance_function);
    
    %construction of the multi-dimensionnal intervals
    I1(:,1)=m1(2,:)';
    I1(:,2)=m1(2,:)'+m1(3,:)';
    I2(:,1)=m2(2,c)';
    I2(:,2)=m2(2,c)'+m2(3,c)';
    
    for i=1:size(I1,1)
        if( I1(i,2)==I1(i,1) && I2(i,2)==I2(i,1) ) %if both intervals are empty (no temporal informations)
            I1(i,:)=I2(i,:); %Comme ca !! < supprime l'information temporelle
        elseif( I1(i,2)==I1(i,1) )
            I1(i,:)=I2(i,:);
        elseif ( I2(i,2)==I2(i,1) )
            I2(i,:)=I1(i,:);
        %Pas de else : cas normal traite par defaut
        end
    end
    
    dist = distfn(I1, I2);
    if( dist < mindist)
        mindist = dist;
    end
    
    % Now, if possible, we modify the matching alignment
    % to force the algorithm to find another one.
    % Just one position is modified, the most possible on the right
    % If not possible, test=0 and it stops the while
    test=0;
    for i=size(c,1):-1:1
        if( c(i) < (size(m2,2)-i+1) )
            c(i)=c(i)+1;
            test=1;
            break
        end
    end
    
end

if( mindist < threshold )
    b=1;
end

