%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This file is part of QTempIntMiner.
%
% QTempIntMiner is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% QTempIntMiner is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% Affero GNU General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with QTempIntMiner.  If not, see <http://www.gnu.org/licenses/>.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%% Test script
% This script generate examples from a pattern prototype, then it computes
% the example to learn a frequent pattern. The protype can be compared with
% the learned pattern.
%
% Author : Guyet T. <thomas.guyet@irisa.fr>


%% EXAMPLE GENERATION OPTIONS
genExamples=1;      % put 0 if you don't want to regenerate the examples
tempDistinction=0;  % generate examples based on two prototypes symbolically identical, but temporally differents (if examples are regenerated)
NbreSymb=5;         % Number of avalaible symbols
NbreEx=100;         % Number of examples
propBruit=0.2;		% propBruit (in [0,1]) is the rate of totally random generated examples.
bruitStruct=0.4;      % structural noise (0 if none)
scenLength=15;      % default duration of the sequence
defaultVar=0.3;     % temporal noise for positions (0 if none)
defaultLVar=0.3;    % temporal noise for duration (0 if none)

%% ALGORITHM OPTIONS
global d;           % points generation density (used by the EM approximation of the histogram)
d=10;
f=0.1;             % frequency threshold
tM=6;				% maximum size of the expected pattern

global TPC_algo;
TPC_algo='Separate'; %Clustering algorithm used for temporal constraint construction
StartLevel_TPruning=0;

global distance_function;
distance_function = 'Hausdorff'; % distance used by KMeans and AP algorithm (must be the name of a function with to parameters I1 and I2, see examples function)

global nbClusterMax;
nbClusterMax=4;     % maximum number of clusters in the EM
threshold=2;          % pattern similarity threshold
global prune_super_explored; %prefixspan "intelligent" pruning
prune_super_explored=true;

global nbClassif; %Compute the total number of classification that have been required !
nbClassif=0;

%% DRAWING OPTIONS
drawExamples=1;         % drawing option : 1 draw examples, 0 no figures
drawResults=1;          % drawing option : 1 draw results, 0 no figures

cmap=hsv(2*NbreSymb+1);         % Colors of the symbols
%cmap=gray(2*NbreSymb+1);       % If you want grey level symbols
cmap=cmap(1:2:2*NbreSymb+1,:);  % To better separate the colors

%% Examples generation
if(genExamples)
    %Prototype choice

    % Prototype with twin symbols
    P=[1  ,2  ,3  ,2 ;...
       2  ,5  ,8  ,12;...
       defaultVar,defaultVar,defaultVar,defaultVar;...
       1  ,0.5, 2  ,0.5;...
       defaultLVar,defaultLVar,defaultLVar,defaultLVar];
    % Prototype with same symbolic pattern but distinct time constraints
    P2=[1  ,2  ,3  ,2 ;...
        1  ,3  ,8  ,12;...
        defaultVar,defaultVar,defaultVar,defaultVar;...
        1  ,0.5, 2  ,0.5;...
        defaultLVar,defaultLVar,defaultLVar,defaultLVar];

    Taille=size(P,2);

    Ex=cell([NbreEx 1]);
    for i=1:NbreEx
        if(rand(1)>propBruit)
            if(tempDistinction)
                %Generate an example based on P or P2
                if(rand(1)>0.5)
                    Ex{i}=GenScen(P,NbreSymb,bruitStruct);
                else
                    Ex{i}=GenScen(P2,NbreSymb,bruitStruct);
                end
            else
                %Generate an example based on P
                Ex{i}=GenScen(P,NbreSymb,bruitStruct);
            end
            
        else
            Ex{i}=GenScenAlea(Taille,NbreSymb,1,scenLength);
        end
    end
    
end

%% Execute the selected algorithm
tic
Pat=QTIPrefixSpan(f, Ex, tM, threshold, StartLevel_TPruning);
%[Pat,patterns]=QTIAPriori(f, Ex, tM, threshold, StartLevel_TPruning);
toc

if(~isempty(Pat))
    %% Select only the longest sequences
    n=size(Pat,2);
    max_size=0;
    argmin=1;
    sel_Pat={};
    for i=1:n
        if(size(Pat{i},2)>max_size)
            max_size = size(Pat{i},2);
            prox_min = inf;
            [r,val]=comparePatterns(Pat{i}, P([1,2,4],:), distance_function, threshold);
            sel_Pat={Pat{i} val};
            if( val<prox_min )
                prox_min=val;
                argmin=1;
            end

        elseif(size(Pat{i},2)==max_size)
            [r,val]=comparePatterns(Pat{i}, P([1,2,4],:), distance_function, threshold);
            sel_Pat=[sel_Pat ; {Pat{i} val}];
            if( val<prox_min )
                prox_min=val;
                argmin=size(sel_Pat,1);
            end
        end
    end
    figure;
    subplot(2,1,1);
    viewScen(P([1,2,4],:),cmap,0);
    xlim([0,scenLength]);
    title('Prototype');
    subplot(2,1,2);
    xlim([0,scenLength]);
    viewScen(sel_Pat{argmin},cmap,0);
    str=sprintf('Best extracted temporal pattern (%f)',prox_min);
    title(str);

    if(tempDistinction)
        max_size=0;
        argmin=1;
        sel_Pat={};
        for i=1:n
            if(size(Pat{i},2)>max_size)
                max_size = size(Pat{i},2);
                prox_min = inf;
                [r,val]=comparePatterns(Pat{i}, P2([1,2,4],:), distance_function, threshold);
                sel_Pat={Pat{i} val};
                if( val<prox_min )
                    prox_min=val;
                    argmin=1;
                end

            elseif(size(Pat{i},2)==max_size)
                [r,val]=comparePatterns(Pat{i}, P2([1,2,4],:), distance_function, threshold);
                sel_Pat=[sel_Pat ; {Pat{i} val}];
                if( val<prox_min )
                    prox_min=val;
                    argmin=size(sel_Pat,1);
                end
            end
        end
        figure;
        subplot(2,1,1);
        viewScen(P2([1,2,4],:),cmap,0);
        xlim([0,scenLength]);
        title('Prototype 2');
        subplot(2,1,2);
        xlim([0,scenLength]);
        viewScen(sel_Pat{argmin},cmap,0);
        str=sprintf('Best extracted temporal pattern (%f)',prox_min);
        title(str);
    end


    %% Drawing result figures (depend on the algorithm result style)
    fprintf('Pat holds %d patterns\n',n);
    if(tempDistinction)
        npat=2;
    else
        npat=1;
    end
    
    if(drawResults && n<30)
        figure;
        subplot(n+npat,1,1);
        viewScen(P([1,2,4],:),cmap,0);
        xlim([0,scenLength]);
        title('Prototype');
        if(tempDistinction)
            subplot(n+npat,1,2);
            viewScen(P2([1,2,4],:),cmap,0);
            xlim([0,scenLength]);
            title('Prototype 2');
        end
        for i=1:n
            subplot(n+npat,1,i+npat);
            viewScen(Pat{i},cmap,0);
            xlim([0,scenLength]);
            [r,val]=comparePatterns(Pat{i}, P([1,2,4],:), distance_function, threshold);
            str=sprintf('Extracted temporal pattern (%f)',val);
            title(str);
        end
    end
end

%% Draw examples (in a new figure)
if(drawExamples)
    figure;
    fprintf('30 first examples drawn among %d examples\n',NbreEx)
    for i=1:min(30,NbreEx)
        subplot( min(30,NbreEx), 1, i);
        viewScen( Ex{i}, cmap, 0);
        xlim([0,scenLength]);
    end
end

